#include "nvfs.h"

void nvfs_memzero_flushcache(char *ptr, size_t len)
{
	memset(ptr, 0, len);
	arch_wb_cache_pmem(ptr, len);
}

uint64_t nvfs_get_random(uint64_t limit)
{
	if (limit < 0x1000000) {
		return prandom_u32() % (uint32_t)limit;
	} else {
		uint64_t result;
		uint64_t v = prandom_u32() | ((uint64_t)prandom_u32() << 32);
		div64_u64_rem(v, limit, &result);
		return result;
	}
}

unsigned long nvfs_find_bit_in_bmp(uint8_t *bitmap, unsigned long start, unsigned long end)
{
	unsigned long o;
	if (!test_bit_le(start, bitmap))
		return start;

	o = find_next_zero_bit_le(bitmap, end, start);
	if (o < end)
		return o;

	o = find_next_zero_bit_le(bitmap, start, 0);
	if (o < start)
		return o;

	return ~0;
}

void nvfs_count_objects(struct nvfs_superblock *nvs, uint64_t *blocks, uint64_t *inodes)
{
	uint64_t g;
	uint64_t b = 0, i = 0;
	for (g = 0; g < nvs->n_groups; g++) {
		struct nvfs_group *pg = p_group(nvs, g);
		b += le32_to_cpu(pg->n_free_blocks);
		i += le32_to_cpu(pg->n_free_inodes);
	}
	*blocks = b;
	*inodes = i;
}
